'use strict'

function isArr(a) {
    return Object.prototype.toString.call(a) === '[object Array]'
}

function isStr(a) {
    return typeof a === 'string'
}

function isFun(a) {
    return typeof a === 'function'
}

function selectOneDeep(tree, filter) {
    filter = filter_formatter(filter)
    var res = selectOneLoop(tree, filter)
    return res
}

function selectOneLoop(list, filter) {
    if (!isArr(list)) {
        return null
    }
    var item = list.find(filter)
    if (item) {
        return item
    } else {
        for (var it of list) {
            if (!isArr(it.children)) {
                continue
            }
            item = selectOneLoop(it.children, filter)
            if (item) {
                return item
            }
        }
    }
}

function selectAllDeep(tree, filter) {
    filter = filter_formatter(filter)
    var result = selectAllLoop(tree, filter)
    return result
}

function selectAllLoop(list, filter) {
    if (!isArr(list)) {
        return []
    }
    var _list = list.filter(filter)
    for(var it of list) {
        if (!isArr(it.children)) {
            continue
        }
        var _ls = selectAllLoop(it.children, filter)
        if (_ls.length) {
            _list = _list.concat(_ls)
        }
    }
    return _list
}

function selectOne(list, filter) {
    if (!isArr(list)) {
        return null
    }
    filter = filter_formatter(filter)
    var one = list.find(filter)
    return one
}

function selectAll(list, filter) {
    if (!isArr(list)) {
        return []
    }
    filter = filter_formatter(filter)
    var _list = list.filter(filter)
    return _list
}

function filter_formatter(filter) {
    if (isStr(filter)) {
        return function (a) {
            return a.name === filter
        }
    }
    if (!isFun(filter)) {
        return function (a) {
            return true
        }
    }
    return filter
}

function El(data) {
    for (let key in data) {
        this[key] = data[key]
    }
}

El.prototype.selectOne = function () {
    var params = this.select_arg_formatter(arguments)
    return selectOne(params.tree, params.filter)
}

El.prototype.selectAll = function () {
    var params = this.select_arg_formatter(arguments)
    return selectAll(params.tree, params.filter)
}

El.prototype.selectOneDeep = function () {
    var params = this.select_arg_formatter(arguments)
    return selectOneDeep(params.tree, params.filter)
}

El.prototype.getAttribute = function (name) {
    return this.attributes[name]
}

El.prototype.selectAllDeep = function () {
    var params = this.select_arg_formatter(arguments)
    return selectAllDeep(params.tree, params.filter)
}

El.prototype.select_arg_formatter = function (args) {
    var tree, filter
    if (args[1] !== undefined) {
        tree = args[0].children
        filter = args[1]
    } else {
        tree = this.children
        filter = args[0]
    }
    return {
        tree: tree,
        filter: filter,
    }
}

module.exports = El